require_relative '../lib/cloudutil/aws.rb'

describe Cloudutil::AWS::Core do
  context 'object initialization' do
    my_env = ENV.to_h

    it 'creates an object using primary env vars' do
      ENV.clear
      ENV['AWS_ACCESS_KEY_ID'] = 'my_primary_access_key'
      ENV['AWS_SECRET_ACCESS_KEY'] = 'my_primary_secret_key'

      aws = Cloudutil::AWS::Core.new

      expect(aws.region).to eq 'us-east-1'
      expect(aws.access_key).to eq 'my_primary_access_key'
      expect(aws.secret_key).to eq 'my_primary_secret_key'
    end

    it 'creates an object using secondary env vars' do
      ENV.clear
      ENV['AWS_ACCESS_KEY'] = 'my_secondary_access_key'
      ENV['AWS_SECRET_KEY'] = 'my_secondary_secret_key'

      aws = Cloudutil::AWS::Core.new

      expect(aws.region).to eq 'us-east-1'
      expect(aws.access_key).to eq 'my_secondary_access_key'
      expect(aws.secret_key).to eq 'my_secondary_secret_key'
    end

    it 'creates an object without env vars' do
      ENV.clear

      aws = Cloudutil::AWS::Core.new

      expect(aws.region).to eq 'us-east-1'
      expect(aws.access_key).to be_nil
      expect(aws.secret_key).to be_nil
    end

    ENV.update(my_env)

    it 'creates an object using implicit hash args' do
      reg = 'us-west-1'
      ak  = 'my_access_key1'
      sk  = 'my_secret_key1'

      aws = Cloudutil::AWS::Core.new(region: reg, access_key: ak, secret_key: sk)

      expect(aws.region).to eq reg
      expect(aws.access_key).to eq ak
      expect(aws.secret_key).to eq sk
    end

    it 'creates an object using explicit hash args' do
      reg = 'us-west-2'
      ak  = 'my_access_key2'
      sk  = 'my_secret_key2'

      aws = Cloudutil::AWS::Core.new(region: reg, access_key: ak, secret_key: sk)

      expect(aws.region).to eq reg
      expect(aws.access_key).to eq ak
      expect(aws.secret_key).to eq sk
    end

    it 'creates an object using sparse hash args' do
      aws = Cloudutil::AWS::Core.new(access_key: 'test_key')

      expect(aws.region).to eq 'us-east-1'
      expect(aws.access_key).to eq 'test_key'
      expect(aws.secret_key).to_not be_nil if ENV['AWS_SECRET_ACCESS_KEY']
    end

    it 'creates an object using implicit array args' do
      reg = 'us-west-1'
      ak  = 'my_access_key3'
      sk  = 'my_secret_key3'

      aws = Cloudutil::AWS::Core.new(region: reg, access_key: ak, secret_key: sk)

      expect(aws.region).to eq reg
      expect(aws.access_key).to eq ak
      expect(aws.secret_key).to eq sk
    end

    it 'creates an object using explicit array args' do
      reg = 'us-west-2'
      ak  = 'my_access_key4'
      sk  = 'my_secret_key4'

      aws = Cloudutil::AWS::Core.new(region: reg, access_key: ak, secret_key: sk)

      expect(aws.region).to eq reg
      expect(aws.access_key).to eq ak
      expect(aws.secret_key).to eq sk
    end

    it 'creates an object using sparse array args' do
      aws = Cloudutil::AWS::Core.new('us-west-2')

      expect(aws.region).to eq 'us-west-2'
      expect(aws.access_key).to_not be_nil if ENV['AWS_ACCESS_KEY_ID']
      expect(aws.secret_key).to_not be_nil if ENV['AWS_SECRET_ACCESS_KEY']
    end
  end
end
