require_relative '../lib/cloudutil/aws.rb'

describe Cloudutil::AWS::SNS do
  context 'object initialization' do
    it 'creates an object using primary env vars' do
      ENV.clear
      ENV['AWS_ACCESS_KEY_ID'] = 'my_primary_access_key'
      ENV['AWS_SECRET_ACCESS_KEY'] = 'my_primary_secret_key'

      aws = Cloudutil::AWS::SNS.new

      expect(aws.region).to eq 'us-east-1'
      expect(aws.access_key).to eq 'my_primary_access_key'
      expect(aws.secret_key).to eq 'my_primary_secret_key'
    end

    it 'creates an object using implicit hash args' do
      reg = 'us-west-1'
      ak  = 'my_access_key1'
      sk  = 'my_secret_key1'

      aws = Cloudutil::AWS::SNS.new(region: reg, access_key: ak, secret_key: sk)

      expect(aws.region).to eq reg
      expect(aws.access_key).to eq ak
      expect(aws.secret_key).to eq sk
    end
  end

  context 'resolving SNS topic ARNs' do
    # Need to do this to read AWS creds from env
    my_env = ENV.to_h
    before(:each) do
      @aws = Cloudutil::AWS::SNS.new(access_key: my_env['AWS_ACCESS_KEY_ID'], secret_key: my_env['AWS_SECRET_ACCESS_KEY'])
    end

    it 'returns the given value if it looks like a topic ARN' do
      data = 'arn:aws:sns:us-east-1:my-topic'
      arn = @aws.resolve_topic_arn(data)
      expect(arn).to eq data
    end

    it 'returns a known topic ARN, case-sensitive', account_specific: true do
      arn = @aws.resolve_topic_arn('storagegateway', false)

      expect(arn).to_not be_nil
      expect(arn).to match(/^arn:aws:sns:.*:storagegateway$/)
    end

    it 'returns a known topic ARN, case-insensitive', account_specific: true do
      arn = @aws.resolve_topic_arn('sTOraGEgaTEwaY')

      expect(arn).to_not be_nil
      expect(arn).to match(/^arn:aws:sns:.*:storagegateway$/)
    end

    it 'returns nil for no case-sensitive match', account_specific: true do
      arn = @aws.resolve_topic_arn('sTOraGEgaTEwaY', false)
      expect(arn).to be_nil
    end

    it 'returns nil for a bogus topic', account_specific: true do
      arn = @aws.resolve_topic_arn('bogus-topic')
      expect(arn).to be_nil
    end
  end
end
